'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var ReactDOMServer = _interopDefault(require('react-dom/server'));
var paramCase = _interopDefault(require('param-case'));
var includes = _interopDefault(require('lodash.includes'));
var uniqueId = _interopDefault(require('lodash.uniqueid'));
var reactTransitionGroup = require('react-transition-group');
var get = _interopDefault(require('lodash.get'));
var classnames = _interopDefault(require('classnames'));
var React = require('react');
var React__default = _interopDefault(React);
var PropTypes = _interopDefault(require('prop-types'));
var merge = _interopDefault(require('lodash.merge'));

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var defaultTheme = {
  prefix: '',
  radius: 'rounded',
  spacing: {
    zero: 0,
    sm: 2,
    md: 4,
    lg: 8,
    xl: 12
  },
  container: {
    sm: 'xl',
    md: '3xl',
    lg: '4xl'
  },
  text: {
    size: {
      body: ['sm', 'base', 'lg'],
      title: ['lg', 'xl', '2xl', '3xl', '4xl', '5xl']
    },
    family: {
      body: 'sans',
      subtitle: 'sans',
      title: 'sans'
    }
  },
  brandColors: {
    primary: 'blue-400',
    secondary: 'gray-500',
    success: 'green-400',
    danger: 'red-400',
    warning: 'orange-400',
    info: 'teal-400'
  },
  textColors: {
    body: 'gray-900',
    link: 'blue-600',
    linkDark: 'blue-800',
    emphasis: 'black',
    on: {
      primary: 'white',
      secondary: 'white',
      success: 'white',
      danger: 'white',
      warning: 'black',
      info: 'black',
      dark: 'white'
    }
  },
  surfaceColors: {
    default: 'white',
    dark: 'gray-700',
    light: 'gray-200'
  },
  highlightOffset: 1,
  accentSize: 4,
  breakpoints: {
    sm: '576px',
    md: '768px',
    lg: '992px',
    xl: '1200px'
  }
};

var TailwindTheme = /*#__PURE__*/React__default.createContext(defaultTheme);

const TailwindThemeProvider = ({
  theme,
  children
}) => {
  const mergedTheme = merge({}, defaultTheme, theme);
  return /*#__PURE__*/React__default.createElement(TailwindTheme.Provider, {
    value: mergedTheme
  }, children);
};

TailwindThemeProvider.displayName = "TailwindThemeProvider";
TailwindThemeProvider.propTypes = {
  theme: PropTypes.shape({}),
  children: PropTypes.node
};
TailwindThemeProvider.defaultProps = {
  theme: {},
  children: undefined
};

const withTheme = Component => {
  const WithTheme = /*#__PURE__*/React.forwardRef((props, ref) => /*#__PURE__*/React__default.createElement(TailwindTheme.Consumer, null, theme => /*#__PURE__*/React__default.createElement(Component, _extends({}, props, {
    ref: ref,
    theme: theme
  }))));
  WithTheme.displayName = "WithTheme(" + Component.displayName + ")";
  return WithTheme;
};

const shades = ['100', '200', '300', '400', '500', '600', '700', '800', '900'];
var getColorShade = ((color, offset = 1) => {
  if (!color) {
    return false;
  }

  if (offset === 0) return color;
  let currentColor = color === 'white' ? ['gray', '100'] : color.split('-');
  let shadeOffset = offset;

  if (color === 'white') {
    if (shadeOffset < 1) return color;
    if (shadeOffset === 1) return 'gray-100';
    currentColor = ['gray', '100'];
    if (typeof shadeOffset === 'number') shadeOffset = offset - 1;
  }

  if (currentColor.length === 1) {
    currentColor.push('400');
  }

  if (typeof shadeOffset === 'string') {
    return currentColor[0] + "-" + shadeOffset;
  }

  const shadeIndex = Math.min(Math.max(shades.indexOf(currentColor[1]) + shadeOffset, 0), shades.length - 1);
  const newShade = shades[shadeIndex];
  return currentColor[0] + "-" + newShade;
});

const getArray = value => Array.isArray(value) ? value : [value];

const splitProp = prop => {
  const utility = prop.substring(prop.indexOf(':') + 1);
  return prop.indexOf(':') !== -1 ? {
    utility,
    variant: prop.substring(0, prop.indexOf(':'))
  } : {
    utility
  };
};

const isNegative = value => Number.isNaN(value) ? /^-.+/.test(value) : value < 0;

const removeLeadingDash = value => String(value).replace(/^-/, '');

const createClassName = ({
  utility,
  value,
  variant,
  prefix = ''
}) => {
  return "" + (variant ? variant + ":" : '') + prefix + (isNegative(value) ? '-' : '') + utility + (value !== false && value !== undefined ? "-" + removeLeadingDash(value) : '');
};
var tailwindPropToClassName = ((prop, values, prefix) => {
  const propType = typeof values;
  if (!propType) return '';
  const {
    utility,
    variant
  } = splitProp(prop);

  if (propType === 'boolean') {
    return createClassName({
      utility,
      variant,
      prefix
    });
  }

  if (propType === 'object' && !Array.isArray(values)) {
    return Object.keys(values).map(key => createClassName({
      prefix,
      utility: "" + utility + key,
      variant,
      value: values[key]
    }));
  }

  return getArray(values).map(value => {
    if (value === false || typeof value === 'undefined') {
      return '';
    }

    if (typeof value === 'boolean') {
      return createClassName({
        utility,
        variant,
        prefix
      });
    }

    return createClassName({
      prefix,
      utility,
      variant,
      value: utility !== value ? value : undefined
    });
  }).filter(value => !!value).join(' ');
});

/* eslint-disable react/destructuring-assignment */
const display = {
  block: PropTypes.bool,
  hidden: PropTypes.bool,
  inline: PropTypes.bool,
  inlineBlock: PropTypes.bool,
  table: PropTypes.bool,
  tableCell: PropTypes.bool,
  tableRow: PropTypes.bool
};
const floats = {
  clearfix: PropTypes.bool,
  float: PropTypes.oneOf(['none', 'right', 'left'])
};
const overflow = {
  overflow: PropTypes.oneOf(['hidden', 'auto', 'scroll']),
  overflowX: PropTypes.oneOf(['hidden', 'auto', 'scroll']),
  overflowY: PropTypes.oneOf(['hidden', 'auto', 'scroll'])
};
const position = {
  absolute: PropTypes.bool,
  fixed: PropTypes.bool,
  inset: PropTypes.oneOfType([PropTypes.oneOf(['0', 'y-0', 'x-0', 'auto']), PropTypes.number]),
  top: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  bottom: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  left: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  right: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  relative: PropTypes.bool,
  static: PropTypes.bool,
  sticky: PropTypes.bool
};
const zIndex = {
  z: PropTypes.oneOfType([PropTypes.number, PropTypes.string])
};
const typography = {
  align: PropTypes.string,
  break: PropTypes.oneOf(['words', 'normal']),
  capitalize: PropTypes.bool,
  font: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
  italic: PropTypes.bool,
  text: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
  tracking: PropTypes.string,
  leading: PropTypes.string,
  lineThrough: PropTypes.bool,
  lowercase: PropTypes.bool,
  normalCase: PropTypes.bool,
  noUnderline: PropTypes.bool,
  notItalic: PropTypes.bool,
  truncate: PropTypes.bool,
  underline: PropTypes.bool,
  uppercase: PropTypes.bool,
  whitespace: PropTypes.oneOf(['normal', 'no-wrap', 'pre', 'pre-line', 'pre-wrap'])
};
const backgrounds = {
  bg: PropTypes.oneOfType([PropTypes.string, PropTypes.array])
};
const borders = {
  border: PropTypes.oneOfType([PropTypes.string, PropTypes.bool, PropTypes.array]),
  borderB: PropTypes.number,
  borderL: PropTypes.number,
  borderR: PropTypes.number,
  borderT: PropTypes.number,
  rounded: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  roundedB: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  roundedBl: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  roundedBr: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  roundedL: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  roundedR: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  roundedT: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  roundedTl: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  roundedTr: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
};
const flexValues = [true, 'row', 'row-reverse', 'col', 'col-reverse', 'no-wrap', 'wrap', 'wrap-reverse', 'initial', 1, 'auto', 'none', 'grow', 'shrink', 'grow-0', 'shrink-0'];
const flexAlignment = ['start', 'center', 'end'];
const flex = {
  content: PropTypes.oneOf([...flexAlignment, 'between', 'around']),
  flex: PropTypes.oneOfType([PropTypes.oneOf(flexValues), PropTypes.arrayOf(PropTypes.oneOf(flexValues))]),
  inlineFlex: PropTypes.bool,
  items: PropTypes.oneOf([...flexAlignment, 'stretch', 'baseline']),
  self: PropTypes.oneOf([...flexAlignment, 'auto', 'stretch']),
  justify: PropTypes.oneOf([...flexAlignment, 'between', 'around'])
};
const spacingShape = {
  t: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  r: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  b: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  l: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  x: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  y: PropTypes.oneOfType([PropTypes.string, PropTypes.number])
};
const spacing = {
  m: PropTypes.oneOfType([PropTypes.string, PropTypes.number, PropTypes.shape(spacingShape)]),
  nm: PropTypes.oneOfType([PropTypes.string, PropTypes.number, PropTypes.shape(spacingShape)]),
  p: PropTypes.oneOfType([PropTypes.string, PropTypes.number, PropTypes.shape(spacingShape)])
};
const sizing = {
  h: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  maxH: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  minH: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  maxW: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  minW: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  w: PropTypes.oneOfType([PropTypes.string, PropTypes.number])
};
const misc = {
  opacity: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  shadow: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  list: PropTypes.oneOf(['none', 'list', 'decimal']),
  outline: PropTypes.string,
  appearance: PropTypes.string,
  select: PropTypes.string,
  pointerEvents: PropTypes.string,
  fill: PropTypes.string
};
const plugins = {
  visuallyHidden: PropTypes.bool,
  visuallyHiddenFocusable: PropTypes.bool,
  container: PropTypes.bool
};
const propTypes = { ...display,
  ...floats,
  ...overflow,
  ...position,
  ...zIndex,
  ...typography,
  ...backgrounds,
  ...borders,
  ...flex,
  ...spacing,
  ...sizing,
  ...misc,
  ...plugins
};
const propVariants = ['hover', 'focus', 'hocus', 'sm', 'md', 'lg', 'xl'];
var tailwindProps = [...Object.keys(propTypes), ...propVariants.reduce((variantProps, variant) => [...variantProps, ...Object.keys(propTypes).map(prop => prop + "-" + variant)], [])];

/* eslint-disable react/destructuring-assignment */

const hasUpperCase = str => str.toLowerCase() !== str;

var getTailwindClassNames = ((props, {
  ignore = [],
  prefix
} = {}) => !!props && Object.keys(props).reduce((twClasses, key) => {
  if (ignore.includes(key) || props[key] === false || typeof props[key] === 'undefined') return twClasses;
  let type = key.indexOf('-') > 0 ? key.substring(0, key.indexOf('-')) : key;
  const variant = key.indexOf('-') > 0 ? key.substring(key.indexOf('-') + 1) : key;
  if (!tailwindProps.includes(type)) return twClasses;

  if (hasUpperCase(type)) {
    type = paramCase(type);
  }

  if (propVariants.includes(variant)) {
    if (variant === 'hocus') {
      return [...twClasses, tailwindPropToClassName("hover:" + type, props[key], prefix), tailwindPropToClassName("focus:" + type, props[key], prefix)];
    }

    return [...twClasses, tailwindPropToClassName(variant + ":" + type, props[key], prefix)];
  }

  return [...twClasses, tailwindPropToClassName(type, props[key], prefix)];
}, []));

var getAsArray = (value => {
  if (!value) return [];
  return Array.isArray(value) ? value : [value];
});

var filterProps = ((componentProps, filterList) => Object.keys(componentProps).reduce((newProps, prop) => {
  if (includes(filterList, prop)) {
    return newProps;
  }

  return { ...newProps,
    [prop]: componentProps[prop]
  };
}, {}));

var getUniqueID = (prefix => uniqueId(prefix + "-"));

var styleProps = ((style, bgImage) => bgImage ? { ...style,
  ...bgImage
} : style);

const withTheme$1 = (Component, {
  inState,
  ...rest
}) => {
  const WithTransition = componentProps => /*#__PURE__*/React__default.createElement(reactTransitionGroup.Transition, _extends({
    in: get(componentProps, inState),
    timeout: 0
  }, rest), state => /*#__PURE__*/React__default.createElement(Component, _extends({}, componentProps, {
    transition: state
  })));

  WithTransition.displayName = "WithTransition(" + Component.displayName + ")";
  return WithTransition;
};

const withTailwind = (Component, {
  ignore = []
} = {}) => {
  const WithTailwind = /*#__PURE__*/React.forwardRef(({
    className,
    ...props
  }, ref) => /*#__PURE__*/React__default.createElement(Component, _extends({}, filterProps(props, tailwindProps.filter(prop => !ignore.includes(prop))), {
    className: classnames([getTailwindClassNames(props, {
      ignore
    }), className]),
    ref: ref
  })));
  WithTailwind.displayName = "WithTailwind(" + Component.displayName + ")";
  WithTailwind.propTypes = {
    className: PropTypes.string,
    ...propTypes
  };
  WithTailwind.defaultProps = {
    className: undefined
  };
  return WithTailwind;
};

var useThemeValue = ((prefix, value, userClassNames = '') => {
  if (userClassNames.includes(prefix)) {
    return false;
  }

  return tailwindPropToClassName(prefix, value);
});

const Base = /*#__PURE__*/React.forwardRef(({
  theme,
  is,
  children,
  className,
  bgImage,
  focusable,
  innerRef,
  style,
  ...rest
}, ref) => {
  const Component = is;
  return /*#__PURE__*/React__default.createElement(Component, _extends({}, filterProps(rest, tailwindProps), {
    className: classnames(getTailwindClassNames({ ...rest,
      'outline-focus': 'none',
      'shadow-focus': 'none'
    }, {
      prefix: theme.prefix
    }), className),
    style: styleProps(style, bgImage),
    ref: innerRef || ref
  }), children);
});
Base.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  className: PropTypes.string,
  innerRef: PropTypes.oneOfType([PropTypes.func, PropTypes.object]),
  ...propTypes
};
Base.defaultProps = {
  is: 'div',
  children: undefined,
  className: undefined,
  innerRef: undefined
};
var Base$1 = withTheme(Base);

const Box = /*#__PURE__*/React.forwardRef(({
  is,
  children,
  inline,
  inlineBlock,
  ...rest
}, ref) => {
  const el = is === 'div' && (inline || inlineBlock) ? 'span' : is;
  return /*#__PURE__*/React__default.createElement(Base$1, _extends({
    is: el,
    inline: inline,
    inlineBlock: inlineBlock,
    ref: ref
  }, rest), children);
});
Box.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  inline: PropTypes.bool,
  inlineBlock: PropTypes.bool
};
Box.defaultProps = {
  is: 'div',
  children: undefined,
  inline: false,
  inlineBlock: false
};

const Flex = /*#__PURE__*/React.forwardRef(({
  is,
  children,
  inline,
  inlineFlex,
  col,
  reverse,
  wrap,
  wrapReverse,
  ...rest
}, ref) => {
  const el = is === 'div' && (inline || inlineFlex) ? 'span' : is;
  const flex = [true];

  if (col) {
    flex.push(reverse ? 'col-reverse' : 'col');
  } else if (reverse) {
    flex.push('row-reverse');
  }

  if (wrap || wrapReverse) {
    flex.push(wrap ? 'wrap' : 'wrap-reverse');
  }

  return /*#__PURE__*/React__default.createElement(Box, _extends({
    is: el,
    flex: flex,
    inlineFlex: inline || inlineFlex,
    ref: ref
  }, rest), children);
});
Flex.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  inline: PropTypes.bool,
  inlineFlex: PropTypes.bool,
  col: PropTypes.bool,
  reverse: PropTypes.bool,
  wrap: PropTypes.bool,
  wrapReverse: PropTypes.bool
};
Flex.defaultProps = {
  is: 'div',
  children: undefined,
  inline: false,
  inlineFlex: false,
  col: false,
  reverse: false,
  wrap: false,
  wrapReverse: false
};

const Image = /*#__PURE__*/React.forwardRef(({
  is,
  children,
  aspectRatio,
  bg,
  w,
  text,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Box, {
  ref: ref,
  relative: true,
  w: w,
  text: text
}, /*#__PURE__*/React__default.createElement(Box, {
  bg: bg,
  style: {
    paddingBottom: 100 / aspectRatio + "%"
  }
}), /*#__PURE__*/React__default.createElement(Base$1, _extends({
  is: is,
  absolute: true,
  inset: 0,
  w: "full"
}, rest)), children && /*#__PURE__*/React__default.createElement(Box, {
  absolute: true,
  inset: 0,
  flex: true,
  items: "end"
}, children)));
Image.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  aspectRatio: PropTypes.number,
  bg: propTypes.bg,
  w: propTypes.w,
  text: propTypes.text
};
Image.defaultProps = {
  is: 'img',
  aspectRatio: 1,
  children: undefined,
  bg: 'gray-light',
  w: 'full',
  text: undefined
};

const Text = /*#__PURE__*/React.forwardRef(({
  children,
  is,
  bold,
  font,
  text,
  color,
  size,
  weight,
  tight,
  loose,
  leading,
  ...rest
}, ref) => {
  const fontValue = [...getAsArray(font), bold ? 'bold' : weight];
  const textValue = [...getAsArray(text), color, size];
  let leadingValue = leading;

  if (tight || loose) {
    leadingValue = tight ? 'tight' : 'loose';
  }

  return /*#__PURE__*/React__default.createElement(Base$1, _extends({
    is: is,
    font: fontValue.filter(value => !!value),
    text: textValue.filter(value => !!value),
    leading: leadingValue,
    ref: ref
  }, rest), children);
});
Text.propTypes = {
  children: PropTypes.node,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  font: propTypes.font,
  text: propTypes.text,
  bold: PropTypes.bool,
  color: PropTypes.string,
  size: PropTypes.string,
  weight: PropTypes.string,
  leading: propTypes.leading,
  tight: PropTypes.bool,
  loose: PropTypes.bool
};
Text.defaultProps = {
  children: undefined,
  is: 'span',
  font: undefined,
  text: undefined,
  bold: false,
  color: undefined,
  size: undefined,
  weight: undefined,
  leading: 'normal',
  tight: false,
  loose: false
};

const focusableElements = ['input', 'select', 'textarea', 'button', 'a'];
const Touchable = /*#__PURE__*/React.forwardRef(({
  is,
  children,
  tabIndex,
  disabled,
  onTouch,
  ...rest
}, ref) => {
  const isSemantic = focusableElements.includes(is);

  const handleKeyPress = e => {
    if (onTouch && (e.key && (e.key === 'Enter' || e.key === ' ' || e.key === 'Spacebar') || e.keyCode && (e.keyCode === 13 || e.keyCode === 32))) {
      e.preventDefault();
      onTouch(e);
    }
  };

  return /*#__PURE__*/React__default.createElement(Base$1, _extends({
    is: is,
    cursor: "pointer",
    pointerEvents: disabled ? 'none' : undefined,
    focusable: true,
    role: !isSemantic ? 'button' : undefined,
    tabIndex: tabIndex || (!isSemantic && !disabled ? 0 : undefined),
    opacity: disabled ? 50 : undefined,
    disabled: disabled,
    "aria-disabled": disabled || undefined,
    onClick: onTouch,
    onKeyPress: !isSemantic && !disabled ? handleKeyPress : undefined,
    ref: ref
  }, rest), children);
});
Touchable.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  disabled: PropTypes.bool,
  tabIndex: PropTypes.number,
  onTouch: PropTypes.func
};
Touchable.defaultProps = {
  is: 'button',
  children: undefined,
  disabled: false,
  tabIndex: undefined,
  onTouch: undefined
};

const Button = /*#__PURE__*/React.forwardRef(({
  theme,
  is,
  children,
  color,
  type,
  buttonStyle,
  disabled,
  large,
  small,
  fullWidth,
  bg,
  text,
  border,
  brand,
  ...rest
}, ref) => {
  const props = {
    border: border || [true, 'transparent'],
    leading: 'tight',
    p: {
      x: theme.spacing.md,
      y: theme.spacing.sm
    },
    rounded: theme.radius
  };

  if (large) {
    props.p = {
      x: theme.spacing.lg,
      y: theme.spacing.md
    };
  } else if (small) {
    props.p = {
      x: theme.spacing.sm,
      y: theme.spacing.sm / 2
    };
  }

  switch (buttonStyle) {
    case 'fill':
      props.bg = brand ? theme.brandColors[brand] : bg;
      props.text = brand ? theme.textColors.on[brand] : text;
      props['bg-hocus'] = getColorShade(brand ? theme.brandColors[brand] : bg, theme.highlightOffset);
      break;

    case 'outline':
      // eslint-disable-next-line react/prop-types
      props.border.push(brand ? theme.brandColors[brand] : border);
      props.text = brand ? theme.brandColors[brand] : border;
      props['bg-hocus'] = brand ? theme.brandColors[brand] : border;
      props['text-hocus'] = brand ? theme.textColors.on[brand] : text;
      break;

    case 'text':
      props.text = brand ? theme.brandColors[brand] : text;
      props['bg-hocus'] = "" + getColorShade(brand ? theme.brandColors[brand] : text, '100');
      break;

    case 'link':
      props.rounded = undefined;
      props.leading = 'normal';
      props.p = 0;
      props.underline = true;
      props.text = brand ? theme.brandColors[brand] : text;
      props['text-hocus'] = getColorShade(brand ? theme.brandColors[brand] : text, theme.highlightOffset);
      break;

    default:
      break;
  }

  if (is === 'button') {
    props.type = type;
  } else {
    props.role = 'button';
  }

  if (fullWidth) {
    props.w = 'full';
  }

  return /*#__PURE__*/React__default.createElement(Touchable, _extends({
    is: is,
    ref: ref,
    inlineBlock: true
  }, props, {
    disabled: disabled
  }, rest), children);
});
Button.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  color: PropTypes.string,
  type: PropTypes.string,
  buttonStyle: PropTypes.oneOf(['fill', 'outline', 'text', 'link']),
  disabled: PropTypes.bool,
  large: PropTypes.bool,
  small: PropTypes.bool,
  fullWidth: PropTypes.bool,
  brand: PropTypes.string,
  bg: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
  text: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
  border: PropTypes.oneOfType([PropTypes.string, PropTypes.array])
};
Button.defaultProps = {
  is: 'button',
  children: undefined,
  color: 'primary',
  type: 'button',
  buttonStyle: 'fill',
  disabled: false,
  large: false,
  small: false,
  fullWidth: false,
  brand: undefined,
  bg: undefined,
  text: undefined,
  border: undefined
};
var Button$1 = withTheme(Button);

/**
 * @see See [Button](#button-2) for API. Sets `buttonStyle` prop to `fill`.
 */

const FillButton = /*#__PURE__*/React.forwardRef((props, ref) => /*#__PURE__*/React__default.createElement(Button$1, _extends({
  ref: ref
}, props, {
  buttonStyle: "fill"
})));

/**
 * @see See [Button](#button-2) for API. Sets `buttonStyle` prop to `outline`.
 */

const OutlineButton = /*#__PURE__*/React.forwardRef((props, ref) => /*#__PURE__*/React__default.createElement(Button$1, _extends({
  ref: ref
}, props, {
  buttonStyle: "outline"
})));

/**
 * @see See [Button](#button-2) for API. Sets `buttonStyle` prop to `text`.
 */

const TextButton = /*#__PURE__*/React.forwardRef((props, ref) => /*#__PURE__*/React__default.createElement(Button$1, _extends({
  ref: ref
}, props, {
  buttonStyle: "text"
})));

/**
 * @see See [Button](#button-2) for API. Sets `buttonStyle` prop to `link`.
 */

const LinkButton = /*#__PURE__*/React.forwardRef((props, ref) => /*#__PURE__*/React__default.createElement(Button$1, _extends({
  ref: ref
}, props, {
  buttonStyle: "link"
})));

const Card = /*#__PURE__*/React.forwardRef(({
  is,
  children,
  theme,
  surface,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Box, _extends({
  is: is,
  overflow: "hidden",
  rounded: theme.radius,
  bg: theme.surfaceColors[surface] || theme.brandColors[surface],
  ref: ref,
  text: surface !== 'default' ? theme.textColors.on[surface] : undefined
}, rest), children));
Card.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  theme: PropTypes.shape({}).isRequired,
  children: PropTypes.node,
  surface: PropTypes.string
};
Card.defaultProps = {
  is: 'section',
  children: undefined,
  surface: 'default'
};
var Card$1 = withTheme(Card);

const CardBody = /*#__PURE__*/React.forwardRef(({
  theme,
  is,
  children,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Box, _extends({
  is: is,
  p: theme.spacing.md,
  ref: ref
}, rest), children));
CardBody.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node
};
CardBody.defaultProps = {
  is: 'div',
  children: undefined
};
var CardBody$1 = withTheme(CardBody);

const CardFooter = /*#__PURE__*/React.forwardRef(({
  is,
  children,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Flex, _extends({
  is: is,
  reverse: true,
  items: "end"
}, rest), React__default.Children.map(children, child => /*#__PURE__*/React__default.cloneElement(child, {
  rounded: 'none'
}))));
CardFooter.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node
};
CardFooter.defaultProps = {
  is: 'div',
  children: undefined
};

const Container = /*#__PURE__*/React.forwardRef(({
  theme,
  is,
  children,
  leftAlign,
  padding,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Box, _extends({
  is: is,
  m: !leftAlign ? {
    x: 'auto'
  } : undefined,
  p: padding ? {
    x: theme.spacing.md
  } : undefined,
  container: true
}, rest), children));
Container.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  leftAlign: PropTypes.bool,
  padding: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
};
Container.defaultProps = {
  is: 'div',
  children: undefined,
  leftAlign: false,
  padding: false
};
var Container$1 = withTheme(Container);

const Title = /*#__PURE__*/React.forwardRef(({
  theme,
  children,
  is,
  size,
  subtitle,
  flush,
  level,
  ...rest
}, ref) => {
  const hLevel = level || Math.max(7 - size, 1);
  const element = is || "h" + hLevel;
  let ariaProps = {};

  if (!subtitle && element !== 'string' && !/h[1-6]/i.test(element)) {
    ariaProps = {
      role: 'heading',
      'aria-level': hLevel
    };
  }

  return /*#__PURE__*/React__default.createElement(Text, _extends({
    is: element
  }, ariaProps, {
    leading: "tight",
    font: [theme.text.family[subtitle ? 'subtitle' : 'title'], subtitle ? 'medium' : 'bold'],
    ref: ref,
    text: [theme.text.size.title[size - 1], subtitle ? theme.textColors.body : theme.textColors.emphasis],
    m: !flush ? {
      b: theme.spacing.md
    } : undefined
  }, rest), children);
});
Title.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  children: PropTypes.node,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  size: PropTypes.number,
  subtitle: PropTypes.bool,
  flush: PropTypes.bool,
  level: PropTypes.number
};
Title.defaultProps = {
  children: undefined,
  is: undefined,
  size: 4,
  subtitle: false,
  flush: false,
  level: undefined
};
var Title$1 = withTheme(Title);

/**
 * @see See [Title](#title) for prop type definitions. Sets `subtitle` to `true`.
 */

const Subtitle = /*#__PURE__*/React.forwardRef((props, ref) => /*#__PURE__*/React__default.createElement(Title$1, _extends({
  ref: ref
}, props, {
  subtitle: true
})));

const Paragraph = /*#__PURE__*/React.forwardRef(({
  theme,
  children,
  is,
  size,
  lead,
  brand,
  paragraph,
  text,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Text, _extends({
  is: is,
  ref: ref,
  text: [(size || lead) && theme.text.size.body[(lead ? theme.text.size.body.length : size) - 1], ...getAsArray(text)],
  m: {
    b: theme.spacing.md
  }
}, rest), children));
Paragraph.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  children: PropTypes.node,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  size: PropTypes.number,
  lead: PropTypes.bool,
  brand: PropTypes.bool,
  paragraph: PropTypes.bool,
  text: propTypes.text
};
Paragraph.defaultProps = {
  children: undefined,
  is: 'p',
  size: undefined,
  paragraph: false,
  lead: false,
  brand: false,
  text: undefined
};
var Paragraph$1 = withTheme(Paragraph);

const BrandText = /*#__PURE__*/React.forwardRef(({
  theme,
  textOnly,
  type,
  ...rest
}, ref) => {
  const alertProps = !textOnly ? {
    bg: getColorShade(theme.brandColors[type], '100'),
    border: ["l-" + theme.accentSize, theme.brandColors[type]],
    p: {
      x: theme.spacing.md,
      y: theme.spacing.sm
    },
    text: theme.textColors.body,
    rounded: 'r'
  } : {};
  return /*#__PURE__*/React__default.createElement(Paragraph$1, _extends({}, rest, {
    brand: true,
    ref: ref,
    rounded: theme.radius,
    text: getColorShade(theme.brandColors[type], 1),
    m: {
      b: theme.spacing.sm
    }
  }, alertProps));
});
BrandText.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  textOnly: PropTypes.bool,
  type: PropTypes.string.isRequired
};
BrandText.defaultProps = {
  is: 'p',
  textOnly: false
};
var BrandText$1 = withTheme(BrandText);

/**
 * @see See [BrandText](#brandtext) for API. Sets `type` prop to `danger`.
 */

const DangerText = /*#__PURE__*/React.forwardRef((props, ref) => /*#__PURE__*/React__default.createElement(BrandText$1, _extends({
  ref: ref
}, props, {
  type: "danger"
})));

/**
 * @see See [BrandText](#brandtext) for API. Sets `type` prop to `info`.
 */

const InfoText = /*#__PURE__*/React.forwardRef((props, ref) => /*#__PURE__*/React__default.createElement(BrandText$1, _extends({
  ref: ref
}, props, {
  type: "info"
})));

/**
 * @see See [BrandText](#brandtext) for API. Sets `type` prop to `warning`.
 */

const WarningText = /*#__PURE__*/React.forwardRef((props, ref) => /*#__PURE__*/React__default.createElement(BrandText$1, _extends({
  ref: ref
}, props, {
  type: "warning"
})));

const ContentTitle = /*#__PURE__*/React.forwardRef(({
  content: {
    id
  },
  visuallyHidden,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Title$1, _extends({
  id: id,
  visuallyHidden: visuallyHidden
}, rest)));
ContentTitle.propTypes = {
  content: PropTypes.shape({
    id: PropTypes.string
  }),
  visuallyHidden: PropTypes.bool
};
ContentTitle.defaultProps = {
  content: {},
  visuallyHidden: false
};

class ContentBlock extends React.PureComponent {
  constructor(props) {
    super(props);
    this.id = getUniqueID('content');
  }

  render() {
    const {
      theme,
      is,
      children,
      ...rest
    } = this.props;
    return /*#__PURE__*/React__default.createElement(Box, _extends({
      is: is,
      p: theme.spacing.md,
      "aria-labelledby": this.id
    }, rest), React__default.Children.map(children, (child, index) => {
      if (child.type === ContentTitle) {
        return /*#__PURE__*/React__default.cloneElement(child, {
          content: {
            id: this.id
          }
        });
      }

      if (index === React__default.Children.count(children) - 1) {
        return /*#__PURE__*/React__default.cloneElement(child, {
          m: {
            b: 0
          }
        });
      }

      return child;
    }));
  }

}

ContentBlock.displayName = "ContentBlock";
ContentBlock.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node
};
ContentBlock.defaultProps = {
  is: 'section',
  children: undefined
};
var ContentBlock$1 = withTheme(ContentBlock);

const Article = /*#__PURE__*/React.forwardRef(({
  is,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(ContentBlock$1, _extends({
  is: is,
  ref: ref
}, rest)));
Article.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object])
};
Article.defaultProps = {
  is: 'article'
};

const Aside = /*#__PURE__*/React.forwardRef(({
  is,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(ContentBlock$1, _extends({
  is: is,
  ref: ref
}, rest)));
Aside.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object])
};
Aside.defaultProps = {
  is: 'aside'
};

const Section = /*#__PURE__*/React.forwardRef(({
  is,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(ContentBlock$1, _extends({
  is: is,
  ref: ref
}, rest)));
Section.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object])
};
Section.defaultProps = {
  is: 'section'
};

const Footer = /*#__PURE__*/React.forwardRef(({
  theme,
  is,
  children,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Box, _extends({
  is: is,
  role: "contentinfo",
  p: {
    t: theme.spacing.lg,
    b: theme.spacing.xl
  },
  bg: theme.brandColors.secondary,
  text: theme.textColors.on.secondary
}, rest), /*#__PURE__*/React__default.createElement(Container$1, {
  padding: true
}, children)));
Footer.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node
};
Footer.defaultProps = {
  is: 'footer',
  children: undefined
};
var Footer$1 = withTheme(Footer);

/**
 * @see Renders as [DangerText](#dangertext) component by default
 */

const ErrorText = /*#__PURE__*/React.forwardRef(({
  field: {
    errorId
  },
  is,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Box, _extends({
  is: is,
  id: errorId,
  "aria-live": "assertive",
  ref: ref
}, rest)));
ErrorText.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  field: PropTypes.shape({
    errorId: PropTypes.string
  }),
  size: PropTypes.number
};
ErrorText.defaultProps = {
  is: DangerText,
  field: {},
  size: 1
};

class Field extends React.PureComponent {
  constructor(props) {
    const {
      id
    } = props;
    super(props);
    this.id = id || getUniqueID('field');
  }

  render() {
    const {
      theme,
      is,
      children,
      className,
      hasHelp,
      hasError,
      disabled,
      optionList,
      ...rest
    } = this.props;
    const fieldProps = {
      inputId: this.id + "-input",
      helpId: hasHelp ? this.id + "-help" : undefined,
      errorId: hasError ? this.id + "-error" : undefined,
      labelId: optionList ? this.id + "-label" : undefined,
      invalid: hasError,
      disabled
    };
    return /*#__PURE__*/React__default.createElement(Box, _extends({
      is: is,
      id: this.id,
      m: {
        b: theme.spacing.md
      },
      maxW: "sm"
    }, filterProps(rest, ['id'])), React__default.Children.map(children, child => /*#__PURE__*/React__default.cloneElement(child, {
      field: fieldProps
    })));
  }

}

Field.displayName = "Field";
Field.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  className: PropTypes.string,
  hasHelp: PropTypes.bool,
  hasError: PropTypes.bool,
  disabled: PropTypes.bool,
  id: PropTypes.string,
  optionList: PropTypes.bool
};
Field.defaultProps = {
  is: 'div',
  children: undefined,
  className: undefined,
  hasHelp: false,
  hasError: false,
  disabled: false,
  id: undefined,
  optionList: false
};
var Field$1 = withTheme(Field);

/**
 * @see Renders as [InfoText](#infotext) component by default
 */

const HelpText = /*#__PURE__*/React.forwardRef(({
  is,
  field: {
    helpId
  },
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Box, _extends({
  is: is,
  id: helpId,
  ref: ref
}, rest)));
HelpText.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  field: PropTypes.shape({
    helpId: PropTypes.string
  }),
  size: PropTypes.number
};
HelpText.defaultProps = {
  is: InfoText,
  field: {},
  size: 1
};

const Label = /*#__PURE__*/React.forwardRef(({
  theme,
  is,
  id,
  field: {
    labelId,
    inputId,
    disabled
  },
  children,
  htmlFor,
  optionList,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Text, _extends({
  is: is,
  id: labelId || id,
  inlineBlock: true,
  htmlFor: !optionList ? inputId || htmlFor : undefined,
  m: {
    b: theme.spacing.sm
  },
  ref: ref,
  opacity: disabled ? 50 : undefined,
  weight: "bold"
}, rest), children));
Label.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  id: PropTypes.string,
  field: PropTypes.shape({
    inputId: PropTypes.string,
    disabled: PropTypes.bool
  }),
  children: PropTypes.node,
  htmlFor: PropTypes.string,
  optionList: PropTypes.bool
};
Label.defaultProps = {
  is: 'label',
  id: undefined,
  field: {
    disabled: false
  },
  children: undefined,
  htmlFor: undefined,
  optionList: false
};
var Label$1 = withTheme(Label);

const ExpandMore = props => /*#__PURE__*/React__default.createElement("svg", _extends({
  xmlns: "http://www.w3.org/2000/svg",
  width: "24",
  height: "24",
  viewBox: "0 0 24 24"
}, props), /*#__PURE__*/React__default.createElement("path", {
  d: "M16.59 8.59L12 13.17 7.41 8.59 6 10l6 6 6-6z"
}), /*#__PURE__*/React__default.createElement("path", {
  d: "M0 0h24v24H0z",
  fill: "none"
}));

ExpandMore.displayName = "ExpandMore";
const Select = /*#__PURE__*/React.forwardRef(({
  theme,
  is,
  field,
  children,
  id,
  name,
  type,
  disabled,
  readOnly,
  invalid,
  placeholder,
  options,
  icon,
  ...rest
}, ref) => {
  const describedBy = [field.errorId, field.helpId].filter(by => by);
  const isInvalid = field.invalid || invalid;
  return /*#__PURE__*/React__default.createElement(Box, {
    relative: true,
    m: {
      b: theme.spacing.sm
    },
    ref: ref
  }, /*#__PURE__*/React__default.createElement(Touchable, _extends({
    is: is,
    appearance: "none",
    bg: "white",
    rounded: theme.radius,
    text: theme.textColors.body,
    p: {
      l: theme.spacing.md,
      r: theme.spacing.lg,
      y: theme.spacing.sm
    },
    border: !isInvalid ? true : [true, theme.brandColors.danger],
    w: "full",
    leading: "tight",
    id: field.inputId || id || name,
    name: name,
    type: type,
    disabled: field.disabled || disabled,
    readOnly: readOnly,
    "aria-invalid": isInvalid || undefined,
    "aria-describedby": describedBy.length ? describedBy.join(' ') : undefined
  }, rest), !!placeholder && /*#__PURE__*/React__default.createElement("option", {
    value: ""
  }, placeholder), options.map(option => /*#__PURE__*/React__default.createElement("option", {
    key: name + "-" + option.value,
    value: option.value
  }, option.label))), /*#__PURE__*/React__default.createElement(Box, {
    absolute: true,
    inset: "y-0",
    right: 0,
    flex: true,
    items: "center",
    p: {
      x: theme.spacing.sm
    },
    pointerEvents: "none"
  }, /*#__PURE__*/React__default.createElement(Box, {
    is: icon,
    h: 6,
    w: 6
  })));
});
Select.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  field: PropTypes.shape({
    inputId: PropTypes.string,
    invalid: PropTypes.bool,
    disabled: PropTypes.bool
  }),
  children: PropTypes.node,
  id: PropTypes.string,
  name: PropTypes.string.isRequired,
  type: PropTypes.string,
  disabled: PropTypes.bool,
  readOnly: PropTypes.bool,
  invalid: PropTypes.bool,
  placeholder: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
  icon: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  options: PropTypes.arrayOf(PropTypes.shape({
    label: PropTypes.string,
    value: PropTypes.string
  }))
};
Select.defaultProps = {
  is: 'select',
  field: {},
  children: undefined,
  id: undefined,
  type: 'text',
  disabled: false,
  readOnly: false,
  invalid: false,
  placeholder: 'Please select',
  icon: ExpandMore,
  options: []
};
var Select$1 = withTheme(Select);

const TextInput = /*#__PURE__*/React.forwardRef(({
  theme,
  is,
  field,
  children,
  id,
  name,
  type,
  disabled,
  readOnly,
  invalid,
  ...rest
}, ref) => {
  const describedBy = [field.errorId, field.helpId].filter(by => by);
  const isInvalid = field.invalid || invalid;
  return /*#__PURE__*/React__default.createElement(Touchable, _extends({
    is: is,
    appearance: "none",
    bg: "white",
    rounded: theme.radius,
    text: theme.textColors.body,
    p: {
      x: theme.spacing.md,
      y: theme.spacing.sm
    },
    m: {
      b: theme.spacing.sm
    },
    border: !isInvalid ? true : [true, theme.brandColors.danger],
    w: "full",
    leading: "tight",
    id: field.inputId || id || name,
    name: name,
    type: type,
    disabled: field.disabled || disabled,
    readOnly: readOnly,
    "aria-invalid": isInvalid || undefined,
    "aria-describedby": describedBy.length ? describedBy.join(' ') : undefined,
    ref: ref
  }, rest));
});
TextInput.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  field: PropTypes.shape({
    inputId: PropTypes.string,
    invalid: PropTypes.bool,
    disabled: PropTypes.bool
  }),
  children: PropTypes.node,
  id: PropTypes.string,
  name: PropTypes.string.isRequired,
  type: PropTypes.string,
  disabled: PropTypes.bool,
  readOnly: PropTypes.bool,
  invalid: PropTypes.bool
};
TextInput.defaultProps = {
  is: 'input',
  field: {},
  children: undefined,
  id: undefined,
  type: 'text',
  disabled: false,
  readOnly: false,
  invalid: false
};
var TextInput$1 = withTheme(TextInput);

const TextArea = /*#__PURE__*/React.forwardRef(({
  theme,
  is,
  field,
  children,
  id,
  name,
  disabled,
  readOnly,
  invalid,
  ...rest
}, ref) => {
  const describedBy = [field.errorId, field.helpId].filter(by => by);
  const isInvalid = field.invalid || invalid;
  return /*#__PURE__*/React__default.createElement(Touchable, _extends({
    is: is,
    appearance: "none",
    bg: "white",
    rounded: theme.radius,
    text: theme.textColors.body,
    p: {
      x: theme.spacing.md,
      y: theme.spacing.sm
    },
    m: {
      b: theme.spacing.sm
    },
    border: !isInvalid ? true : [true, theme.brandColors.danger],
    w: "full",
    leading: "tight",
    id: field.inputId || id || name,
    name: name,
    disabled: field.disabled || disabled,
    readOnly: readOnly,
    "aria-invalid": isInvalid || undefined,
    "aria-describedby": describedBy.length ? describedBy.join(' ') : undefined,
    ref: ref
  }, rest));
});
TextArea.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  field: PropTypes.shape({
    inputId: PropTypes.string,
    invalid: PropTypes.bool,
    disabled: PropTypes.bool
  }),
  children: PropTypes.node,
  id: PropTypes.string,
  name: PropTypes.string.isRequired,
  disabled: PropTypes.bool,
  readOnly: PropTypes.bool,
  invalid: PropTypes.bool
};
TextArea.defaultProps = {
  is: 'textarea',
  field: {},
  children: undefined,
  id: undefined,
  disabled: false,
  readOnly: false,
  invalid: false
};
var TextArea$1 = withTheme(TextArea);

class OptionInput extends React.PureComponent {
  constructor(props) {
    super(props);
    const {
      defaultChecked,
      checked
    } = props;
    this.state = {
      checked: defaultChecked || !!checked
    };
    this.inputRef = null;
    this.handleChange = this.handleChange.bind(this);
    this.handleUpdate = this.handleUpdate.bind(this);
  }

  componentDidUpdate(prevProps) {
    this.handleUpdate(prevProps);
  }

  handleChange(e) {
    const {
      checkbox,
      onChange,
      checked
    } = this.props;

    if (checked === undefined) {
      this.setState(({
        checked: checkedState
      }) => ({
        checked: checkbox ? !checkedState : true
      }));
    }

    if (onChange) onChange({ ...e,
      target: this.inputRef.current
    });
    e.preventDefault();
  }

  handleUpdate() {
    const {
      checked
    } = this.props;
    const {
      checked: checkedState
    } = this.state;

    if (checked !== undefined && checked !== checkedState) {
      this.setState({
        checked
      });
    }
  }

  render() {
    const {
      theme,
      name,
      value,
      label,
      hideLabel,
      checkbox,
      id,
      ...rest
    } = this.props;
    const {
      checked
    } = this.state;
    this.inputRef = /*#__PURE__*/React__default.createRef();
    return /*#__PURE__*/React__default.createElement(Touchable, _extends({
      is: Label$1,
      flex: true,
      items: "center",
      onTouch: this.handleChange
    }, rest), /*#__PURE__*/React__default.createElement(Box, {
      is: "input",
      id: id,
      visuallyHidden: true,
      name: name,
      type: checkbox ? 'checkbox' : 'radio',
      value: value,
      checked: checked,
      tabIndex: "-1",
      innerRef: this.inputRef,
      onChange: () => {}
    }), /*#__PURE__*/React__default.createElement(Flex, {
      items: "center",
      justify: "center",
      inlineBlock: true,
      rounded: checkbox ? theme.radius : 'full',
      h: 4,
      w: 4,
      border: [true, checked ? theme.brandColors.primary : false].filter(prop => !!prop),
      bg: checkbox && checked ? theme.brandColors.primary : undefined,
      m: {
        r: theme.spacing.sm
      }
    }, checked && (checkbox ? /*#__PURE__*/React__default.createElement(Box, {
      is: "svg",
      xmlns: "http://www.w3.org/2000/svg",
      viewBox: "0 0 24 24",
      h: 3,
      w: 3,
      text: "white",
      fill: "current"
    }, /*#__PURE__*/React__default.createElement("path", {
      d: "M0 0h24v24H0z",
      fill: "none"
    }), /*#__PURE__*/React__default.createElement("path", {
      d: "M9 16.17L4.83 12l-1.42 1.41L9 19 21 7l-1.41-1.41z"
    })) : /*#__PURE__*/React__default.createElement(Box, {
      inlineBlock: true,
      rounded: "full",
      h: 2,
      w: 2,
      bg: theme.brandColors.primary
    }))), /*#__PURE__*/React__default.createElement(Box, {
      inlineBlock: true,
      visuallyHidden: hideLabel,
      leading: "tight",
      font: "normal"
    }, label));
  }

}

OptionInput.displayName = "OptionInput";
OptionInput.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  name: PropTypes.string.isRequired,
  value: PropTypes.string.isRequired,
  label: PropTypes.string.isRequired,
  hideLabel: PropTypes.bool,
  defaultChecked: PropTypes.bool,
  checked: PropTypes.bool,
  onChange: PropTypes.func,
  checkbox: PropTypes.bool,
  id: PropTypes.string
};
OptionInput.defaultProps = {
  hideLabel: false,
  defaultChecked: undefined,
  checked: undefined,
  onChange: undefined,
  checkbox: false,
  id: undefined
};
var OptionInput$1 = withTheme(OptionInput);

const Radio = /*#__PURE__*/React.forwardRef((props, ref) => /*#__PURE__*/React__default.createElement(OptionInput$1, _extends({}, props, {
  checkbox: false,
  ref: ref
})));

const Checkbox = /*#__PURE__*/React.forwardRef((props, ref) => /*#__PURE__*/React__default.createElement(OptionInput$1, _extends({
  ref: ref
}, props, {
  checkbox: true
})));

const List = /*#__PURE__*/React.forwardRef(({
  theme,
  is,
  children,
  padding,
  list,
  inline,
  justified,
  fullWidth,
  ordered,
  listItemIs,
  ...rest
}, ref) => {
  let listStyle = list;

  if (listStyle === true) {
    listStyle = ordered ? 'decimal' : 'disc';
  }

  return /*#__PURE__*/React__default.createElement(Box, _extends({
    is: ordered ? 'ol' : is,
    m: {
      b: theme.spacing.md
    },
    p: listStyle ? {
      l: theme.spacing.md
    } : undefined,
    flex: justified || fullWidth || (inline ? [true, 'wrap', ...rest.flex] : rest.flex),
    justify: justified ? 'between' : undefined,
    list: listStyle,
    ref: ref
  }, rest), React__default.Children.map(children, child => child && /*#__PURE__*/React__default.createElement(Box, {
    is: listItemIs,
    m: {
      b: padding && !justified && !fullWidth && theme.spacing.sm,
      r: inline && theme.spacing.sm
    },
    flex: fullWidth ? 'grow' : undefined
  }, child)));
});
List.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  padding: PropTypes.bool,
  list: PropTypes.oneOfType([PropTypes.bool, PropTypes.oneOf(['disc', 'decimal'])]),
  inline: PropTypes.bool,
  justified: PropTypes.bool,
  fullWidth: PropTypes.bool,
  ordered: PropTypes.bool,
  listItemIs: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object])
};
List.defaultProps = {
  is: 'ul',
  children: undefined,
  padding: false,
  list: undefined,
  inline: false,
  justified: false,
  fullWidth: false,
  ordered: false,
  listItemIs: 'li'
};
var List$1 = withTheme(List);

class OptionList extends React.PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      checked: []
    };
    this.handleChange = this.handleChange.bind(this);
  }

  handleChange(e) {
    const {
      checkbox
    } = this.props;
    const {
      checked
    } = this.state;
    const {
      value
    } = e.target;

    if (checkbox) {
      return this.setState({
        checked: checked.includes(value) ? checked.filter(checkedVal => checkedVal !== value) : [...checked, value]
      });
    }

    return this.setState({
      checked: [e.target.value]
    });
  }

  render() {
    const {
      theme,
      children,
      name,
      checkbox,
      field,
      invalid,
      ...rest
    } = this.props;
    const {
      checked
    } = this.state;
    const describedBy = [field.labelId, field.errorId, field.helpId].filter(by => by);
    const isInvalid = field.invalid || invalid;
    return /*#__PURE__*/React__default.createElement(List$1, _extends({
      reset: true,
      m: {
        b: 0
      }
    }, rest), React__default.Children.map(children, child => {
      const value = child.props && child.props.value;
      return /*#__PURE__*/React__default.cloneElement(child, {
        id: field.inputId + "-" + value,
        name: checkbox ? name + "[]" : name,
        checked: checked.includes(value),
        onChange: this.handleChange,
        'aria-invalid': isInvalid || undefined,
        'aria-describedby': describedBy.length ? describedBy.join(' ') : undefined
      });
    }));
  }

}

OptionList.displayName = "OptionList";
OptionList.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  children: PropTypes.node,
  name: PropTypes.string.isRequired,
  field: PropTypes.shape({
    inputId: PropTypes.string,
    invalid: PropTypes.bool,
    disabled: PropTypes.bool
  }),
  checkbox: PropTypes.bool,
  invalid: PropTypes.bool
};
OptionList.defaultProps = {
  children: undefined,
  checkbox: false,
  field: {},
  invalid: false
};
var OptionList$1 = withTheme(OptionList);

const getWidthProps = width => {
  if (typeof width === 'object') {
    return Object.keys(width).reduce((props, breakpoint) => {
      const breakpointSuffix = breakpoint === 'def' ? '' : "-" + breakpoint;

      if (width === 'auto') {
        return { ...props,
          ["flex" + breakpointSuffix]: 1
        };
      }

      return { ...props,
        ["w" + breakpointSuffix]: width[breakpoint]
      };
    }, {});
  }

  return width === 'auto' ? {
    flex: 1
  } : {
    w: width
  };
};

const Col = /*#__PURE__*/React.forwardRef(({
  is,
  children,
  w,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Box, _extends({
  is: is
}, getWidthProps(w), rest, {
  ref: ref
}), children));
Col.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  w: PropTypes.oneOfType([PropTypes.string, PropTypes.object])
};
Col.defaultProps = {
  is: 'li',
  children: undefined,
  w: 'full'
};

const Row = /*#__PURE__*/React.forwardRef(({
  is,
  children,
  nowrap,
  gutter,
  theme,
  ...rest
}, ref) => {
  const gutterSpacing = gutter && (gutter === true ? theme.spacing.md : theme.spacing[gutter]);
  return /*#__PURE__*/React__default.createElement(Flex, _extends({
    is: is,
    wrap: !nowrap,
    nm: gutter ? {
      l: gutterSpacing,
      b: !nowrap ? gutterSpacing : undefined
    } : undefined,
    listReset: true,
    ref: ref
  }, rest), gutter ? React__default.Children.map(children, child => /*#__PURE__*/React__default.cloneElement(child, {
    p: {
      l: gutterSpacing
    },
    m: {
      b: gutterSpacing
    }
  })) : children);
});
Row.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  nowrap: PropTypes.bool,
  gutter: PropTypes.oneOfType([PropTypes.string, PropTypes.bool])
};
Row.defaultProps = {
  is: 'ul',
  children: undefined,
  nowrap: false,
  gutter: false
};
var Row$1 = withTheme(Row);

class Header extends React.PureComponent {
  constructor(props) {
    const {
      id,
      screen
    } = props;
    super(props);
    this.state = {
      open: false,
      collapsable: !!screen
    };
    this.mql = null;
    this.id = id || getUniqueID('header');
    this.handleMatch = this.handleMatch.bind(this);
    this.handleToggle = this.handleToggle.bind(this);
  }

  componentDidMount() {
    const {
      theme,
      screen
    } = this.props;

    if (screen && window.matchMedia) {
      this.mql = window.matchMedia("(min-width: " + theme.breakpoints[screen] + ")");
      this.mql.addListener(this.handleMatch);

      if (this.mql.matches) {
        this.handleMatch(this.mql);
      }
    }
  }

  componentWillUnmount() {
    if (this.mql) this.mql.removeListener(this.handleMatch);
  }

  handleToggle(forceState) {
    const {
      open
    } = this.state;
    this.setState({
      open: forceState || !open
    });
  }

  handleMatch(mql) {
    this.setState({
      collapsable: !mql.matches
    });
  }

  render() {
    const {
      open,
      collapsable
    } = this.state;
    const {
      theme,
      is,
      children,
      bg,
      text,
      screen,
      ...rest
    } = this.props;
    const headerProps = {
      id: this.id,
      style: {
        bg: bg || theme.brandColors.primary,
        text: text || theme.textColors.on.primary
      },
      open,
      collapsable,
      onToggle: this.handleToggle,
      screen
    };
    return /*#__PURE__*/React__default.createElement(Box, _extends({
      is: is,
      id: headerProps.id,
      bg: headerProps.style.bg,
      text: headerProps.style.text,
      p: {
        y: theme.spacing.md
      },
      role: "banner"
    }, filterProps(rest, ['id'])), /*#__PURE__*/React__default.createElement(Container$1, {
      is: Flex,
      wrap: true,
      items: "center",
      justify: "between",
      padding: true
    }, React__default.Children.map(children, child => /*#__PURE__*/React__default.cloneElement(child, {
      header: headerProps
    }))));
  }

}

Header.displayName = "Header";
Header.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  bg: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
  text: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
  id: PropTypes.string,
  screen: PropTypes.oneOfType([PropTypes.bool, PropTypes.string])
};
Header.defaultProps = {
  is: 'header',
  children: undefined,
  bg: undefined,
  text: undefined,
  id: undefined,
  screen: 'lg'
};
var Header$1 = withTheme(Header);

const NavBrand = /*#__PURE__*/React.forwardRef(({
  theme,
  header: {
    style,
    screen
  },
  is,
  children,
  ...rest
}, ref) => {
  const responsive = screen ? {
    ["m-" + screen]: {
      r: theme.spacing.lg
    }
  } : {};
  const aria = !(typeof is === 'string' && is.startsWith('h')) ? {
    role: 'heading',
    'aria-level': 1
  } : {};
  return /*#__PURE__*/React__default.createElement(Box, _extends({
    is: is,
    inlineBlock: true,
    flex: [true, 'no-shrink'],
    items: "center",
    h: 12,
    text: style.text || theme.textColors.on.primary,
    ref: ref
  }, responsive, aria, rest), children);
});
NavBrand.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  header: PropTypes.shape({
    style: PropTypes.object,
    screen: PropTypes.oneOfType([PropTypes.bool, PropTypes.string])
  })
};
NavBrand.defaultProps = {
  is: 'div',
  children: undefined,
  header: {
    style: {},
    screen: 'lg'
  }
};
var NavBrand$1 = withTheme(NavBrand);

const NavItem = /*#__PURE__*/React.forwardRef(({
  theme,
  is,
  children,
  header: {
    style,
    screen
  },
  active,
  ...rest
}, ref) => {
  const textColor = style.text || theme.textColors.on.primary;
  const bgColor = style.bg || theme.brandColors.primary;
  const responsive = screen ? {
    ["m-" + screen]: {
      t: 0,
      r: theme.spacing.sm
    }
  } : {};
  return /*#__PURE__*/React__default.createElement(Touchable, _extends({
    is: is,
    focusable: true,
    text: !active ? style.text : style.bg,
    bg: active ? textColor : undefined,
    "bg-hocus": textColor,
    "text-hocus": bgColor,
    p: {
      x: theme.spacing.md,
      y: theme.spacing.sm
    },
    m: {
      t: theme.spacing.sm
    },
    rounded: theme.radius,
    block: true,
    "aria-current": active ? 'page' : undefined,
    ref: ref
  }, responsive, rest), children);
});
NavItem.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  header: PropTypes.shape({
    style: PropTypes.object,
    screen: PropTypes.oneOfType([PropTypes.bool, PropTypes.string])
  }),
  active: PropTypes.bool
};
NavItem.defaultProps = {
  is: 'a',
  children: undefined,
  header: {
    style: {},
    screen: 'lg'
  },
  active: false
};
var NavItem$1 = withTheme(NavItem);

const NavItemWrapper = props => /*#__PURE__*/React__default.createElement("li", _extends({
  role: "none"
}, props));

NavItemWrapper.displayName = "NavItemWrapper";
const NavMenu = /*#__PURE__*/React.forwardRef(({
  theme,
  transition,
  is,
  children,
  header,
  list,
  ...rest
}, ref) => {
  const transitionStyles = {
    entering: {
      maxHeight: '0',
      position: 'absolute'
    },
    entered: {
      maxHeight: '100vh'
    }
  };
  const headingId = header.id + "-menu";
  const responsive = {
    nav: header.screen ? {
      ["w-" + header.screen]: 'auto',
      ["flex-" + header.screen]: true
    } : {},
    menu: header.screen ? {
      ["flex-" + header.screen]: true,
      ["m-" + header.screen]: {
        b: 0
      }
    } : {}
  };
  return /*#__PURE__*/React__default.createElement(Box, _extends({
    is: is,
    overflow: "hidden",
    w: "full",
    flex: "grow",
    items: "center",
    h: !header.collapsable ? 12 : undefined,
    style: header.collapsable ? {
      transition: 'max-height 500ms',
      maxHeight: '0',
      ...transitionStyles[transition]
    } : undefined,
    id: header.id + "-nav",
    "aria-labelledby": headingId,
    "aria-expanded": header.collapsable ? header.open : undefined,
    role: "navigation",
    ref: ref
  }, responsive.nav, rest), /*#__PURE__*/React__default.createElement(Title$1, {
    level: 2,
    id: headingId,
    visuallyHidden: true
  }, "Site menu"), /*#__PURE__*/React__default.createElement(List$1, _extends({
    reset: true,
    flex: "grow",
    role: "menu",
    listItemIs: NavItemWrapper
  }, responsive.menu, list), React__default.Children.map(children, child => child.type === NavItem$1 && /*#__PURE__*/React__default.cloneElement(child, {
    header,
    role: 'menuitem'
  }))), React__default.Children.map(children, child => child.type !== NavItem$1 && child));
});
NavMenu.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  transition: PropTypes.string,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  children: PropTypes.node,
  header: PropTypes.shape({
    open: PropTypes.bool,
    collapsable: PropTypes.bool,
    screen: PropTypes.oneOfType([PropTypes.bool, PropTypes.string])
  }),
  list: PropTypes.shape({})
};
NavMenu.defaultProps = {
  is: 'nav',
  children: undefined,
  header: {
    open: false,
    collapsable: false,
    screen: 'lg'
  },
  transition: 'entering',
  list: {}
};
var NavMenu$1 = withTheme(withTheme$1(NavMenu, {
  inState: 'header.open'
}));

const Bar = () => /*#__PURE__*/React__default.createElement(Box, {
  inlineBlock: true,
  border: "b",
  style: {
    borderColor: 'currentColor'
  }
});

Bar.displayName = "Bar";
const NavToggle = /*#__PURE__*/React.forwardRef(({
  theme,
  children,
  onClick,
  header: {
    onToggle,
    style,
    id,
    screen
  },
  ...rest
}, ref) => {
  const handleClick = e => {
    onToggle();
    if (onClick) onClick(e);
  };

  const responsive = screen ? {
    ["hidden-" + screen]: true
  } : {};
  return /*#__PURE__*/React__default.createElement(Button$1, _extends({
    w: 12,
    h: 12,
    p: 0,
    block: true,
    onClick: handleClick,
    "aria-label": "Open menu",
    "aria-haspopup": "true",
    "aria-controls": id + "-nav",
    text: style.text || theme.textColors.on.primary,
    "bg-hocus": style.text || theme.textColors.on.primary,
    "text-hocus": style.bg || theme.brandColors.primary,
    ref: ref
  }, responsive, rest), children || /*#__PURE__*/React__default.createElement(Box, {
    flex: [true, 'col'],
    items: "stretch",
    justify: "around",
    h: "full",
    p: 3
  }, /*#__PURE__*/React__default.createElement(Bar, null), /*#__PURE__*/React__default.createElement(Bar, null), /*#__PURE__*/React__default.createElement(Bar, null)));
});
NavToggle.propTypes = {
  theme: PropTypes.shape({}).isRequired,
  children: PropTypes.node,
  header: PropTypes.shape({
    onToggle: PropTypes.func.isRequired,
    screen: PropTypes.oneOfType([PropTypes.bool, PropTypes.string]),
    style: PropTypes.object
  }),
  onClick: PropTypes.func
};
NavToggle.defaultProps = {
  children: undefined,
  onClick: undefined,
  header: {
    style: {},
    screen: 'lg'
  }
};
var NavToggle$1 = withTheme(NavToggle);

const Main = /*#__PURE__*/React.forwardRef(({
  children,
  is,
  id,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Box, _extends({
  is: is,
  id: id,
  role: "main"
}, rest), children));
Main.propTypes = {
  children: PropTypes.node,
  id: PropTypes.string,
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object])
};
Main.defaultProps = {
  children: undefined,
  id: 'main',
  is: 'main'
};

const SkipLink = /*#__PURE__*/React.forwardRef(({
  children,
  href,
  ...rest
}, ref) => /*#__PURE__*/React__default.createElement(Box, {
  visuallyHiddenFocusable: true
}, /*#__PURE__*/React__default.createElement(FillButton, _extends({
  is: "a",
  bg: "white",
  text: "black",
  absolute: true,
  rounded: "none",
  href: href,
  ref: ref
}, rest), children || 'Skip to main content')));
SkipLink.propTypes = {
  children: PropTypes.node,
  href: PropTypes.string
};
SkipLink.defaultProps = {
  children: undefined,
  href: '#main'
};

const SiteWrap = /*#__PURE__*/React.forwardRef(({
  is,
  theme,
  children,
  ...rest
}, ref) => {
  let footer;
  React__default.Children.forEach(children, child => {
    const {
      type
    } = child;

    if (type && (type === Footer$1 || type.displayName === Footer.displayName)) {
      footer = child;
    }
  });
  return /*#__PURE__*/React__default.createElement(Flex, _extends({
    is: is,
    col: true,
    minH: "screen",
    leading: "normal",
    font: theme.text.family.body,
    ref: ref,
    text: [theme.text.size.body[1], theme.textColors.body]
  }, rest), /*#__PURE__*/React__default.createElement(Box, {
    flex: ['auto', 'no-shrink']
  }, React__default.Children.map(children, child => {
    if (child === footer) return false;
    return child;
  })), /*#__PURE__*/React__default.createElement(Box, {
    flex: ['auto', 'no-shrink', 'no-grow'],
    m: {
      t: theme.spacing.lg
    }
  }, footer));
});
SiteWrap.propTypes = {
  is: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.object]),
  theme: PropTypes.shape({}).isRequired,
  children: PropTypes.node
};
SiteWrap.defaultProps = {
  is: 'div',
  children: undefined
};
var SiteWrap$1 = withTheme(SiteWrap);



var lib = /*#__PURE__*/Object.freeze({
  Button: Button$1,
  FillButton: FillButton,
  OutlineButton: OutlineButton,
  TextButton: TextButton,
  LinkButton: LinkButton,
  Card: Card$1,
  CardBody: CardBody$1,
  CardFooter: CardFooter,
  Container: Container$1,
  ContentTitle: ContentTitle,
  ContentBlock: ContentBlock$1,
  Article: Article,
  Aside: Aside,
  Section: Section,
  Footer: Footer$1,
  ErrorText: ErrorText,
  Field: Field$1,
  HelpText: HelpText,
  Label: Label$1,
  Select: Select$1,
  TextInput: TextInput$1,
  TextArea: TextArea$1,
  Radio: Radio,
  Checkbox: Checkbox,
  OptionList: OptionList$1,
  Col: Col,
  Row: Row$1,
  Header: Header$1,
  NavBrand: NavBrand$1,
  NavItem: NavItem$1,
  NavMenu: NavMenu$1,
  NavToggle: NavToggle$1,
  List: List$1,
  Main: Main,
  SkipLink: SkipLink,
  Base: Base$1,
  Box: Box,
  Flex: Flex,
  Image: Image,
  Text: Text,
  Touchable: Touchable,
  SiteWrap: SiteWrap$1,
  Subtitle: Subtitle,
  Paragraph: Paragraph$1,
  Title: Title$1,
  BrandText: BrandText$1,
  DangerText: DangerText,
  InfoText: InfoText,
  WarningText: WarningText,
  getColorShade: getColorShade,
  getTailwindClassNames: getTailwindClassNames,
  tailwindProps: tailwindProps,
  propTypes: propTypes,
  propVariants: propVariants,
  tailwindPropToClassName: tailwindPropToClassName,
  createClassName: createClassName,
  withTailwind: withTailwind,
  defaultTheme: defaultTheme,
  TailwindTheme: TailwindTheme,
  TailwindThemeProvider: TailwindThemeProvider,
  withTheme: withTheme,
  useThemeValue: useThemeValue,
  getAsArray: getAsArray,
  filterProps: filterProps,
  getUniqueID: getUniqueID,
  styleProps: styleProps,
  withTransition: withTheme$1
});

var renderComponentsToString = ((projectTheme = {}) => {
  const {
    TailwindThemeProvider: TailwindThemeProvider$$1
  } = lib;
  return ReactDOMServer.renderToStaticMarkup( /*#__PURE__*/React__default.createElement(TailwindThemeProvider$$1, {
    theme: projectTheme
  }, Object.keys(lib).map(key => {
    if (/^[a-z]/.test(key) || key.startsWith('TailwindTheme')) {
      return false;
    }

    const Component = lib[key];
    return /*#__PURE__*/React__default.createElement(Component, null);
  })));
});

const mapObject = (obj, prefix) => Object.keys(obj).map(key => prefix + "-" + obj[key]);

const mapArray = (arr, prefix) => arr.map(value => prefix + "-" + value);

const reduceMultiple = (obj, prefixes, suffixes) => Object.keys(obj).reduce((arr, key) => {
  if (typeof obj[key] === 'object') {
    return [...arr, ...reduceMultiple(obj[key], prefixes)];
  }

  if (suffixes) {
    const baseClass = obj[key].indexOf('-') !== -1 ? obj[key].split('-')[0] : obj[key];
    return [...arr, ...prefixes.reduce((classes, prefix) => [...classes, ...suffixes.map(suffix => suffix !== '' ? "-" + suffix : '').map(suffix => prefix + "-" + baseClass + suffix)], [])];
  }

  return [...arr, ...prefixes.map(prefix => prefix + "-" + obj[key])];
}, []);

const colorPrefixes = ['bg', 'border', 'text', 'hover:bg', 'hover:border', 'hover:text', 'focus:bg', 'focus:border', 'focus:text'];
const colorSuffixes = ['100', '200', '300', '400', '500', '600', '700', '800', '900'];
var getThemeWhitelist = ((projectTheme = {}) => {
  const theme = merge({}, defaultTheme, projectTheme);
  return [theme.radius, ...reduceMultiple(theme.spacing, ['mt', 'mb', 'mr', '-ml', '-mr', 'p-', 'pl-', 'px-', 'py-']), ...mapObject(theme.container, 'max-w'), ...mapArray(theme.text.size.body, 'text'), ...mapArray(theme.text.size.title, 'text'), ...mapObject(theme.text.family, 'font'), ...reduceMultiple(theme.brandColors, colorPrefixes, colorSuffixes), ...reduceMultiple(theme.textColors, colorPrefixes)].filter((className, index, self) => self.indexOf(className) === index);
});

/* eslint-disable no-useless-escape */
const classRegex = /[A-Za-z0-9-_:\/]+/g; // TODO: Array values

const propRegex = {
  singleValue: prop => new RegExp("[ \n\r\t]" + prop + "(?:-[A-Za-z]+)?=[\"'{}][A-Za-z0-9-_:/]+[\"'}]", 'g'),
  booleanValue: prop => new RegExp("[ \n\r\t]" + prop + "(?:-[A-Za-z]+)?(?:[ \n\r]|>)", 'g'),
  objectValue: prop => new RegExp("[ \n\r\t]" + prop + "(?:-[A-Za-z]+)?={{ ?[A-Za-z0-9-_:/ '\",]+ ?}}", 'g')
};
const transformations = {
  'w-auto': 'flex-1'
};
class TailwindReactExtractor {
  static extract(content) {
    let classNames = [];
    tailwindProps.forEach(prop => {
      let matches = [...(content.match(propRegex.singleValue(prop)) || []), ...(content.match(propRegex.booleanValue(prop)) || [])];
      let objMatches = content.match(propRegex.objectValue(prop)) || [];

      if (objMatches.length) {
        objMatches = objMatches.reduce((array, match) => [...array, ...match.replace(/\s/g, '').replace(prop + "=", '').replace(/[{}'"]/g, '').split(',').map(objProp => {
          const classFragments = objProp.split(':');
          const key = classFragments[0];
          return key.length === 1 ? "" + prop + key + "-" + classFragments[1] : "" + prop + (key === 'def' ? '' : "-" + classFragments[0]) + "-" + classFragments[1];
        })], []);
        matches = matches.concat(objMatches);
      }

      classNames = classNames.concat(matches.map(match => match.replace(/[\s"'{}>]/g, '').replace('=', '-').replace(/[A-Z]/g, '-$&').toLowerCase()).reduce((classes, match) => {
        let className = [match];
        propVariants.forEach(variant => {
          if (match.includes(variant)) {
            className = match.replace("-" + variant, '');
            className = variant === 'hocus' ? ["hover:" + className, "focus:" + className] : [variant + ":" + className];
          }
        });

        if ((prop === 'bg' || prop === 'text') && className[0].indexOf(':') === -1) {
          // TODO: Determine if prop value is a colour
          const baseColor = className[0].replace(prop + "-", '');
          const highlightColor = getColorShade(baseColor);
          className = [match, "hover:" + prop + "-" + baseColor, "focus:" + prop + "-" + baseColor, "hover:" + prop + "-" + highlightColor, "focus:" + prop + "-" + highlightColor];

          if (prop === 'bg') {
            className.push("hover:" + prop + "-" + getColorShade(baseColor, '100'), "focus:" + prop + "-" + getColorShade(baseColor, '100'));
          }
        }

        return [...classes, ...className];
      }, [])).map(className => {
        if (className.indexOf(':') !== -1) {
          const classFragments = className.split(':');
          return transformations[classFragments[1]] ? classFragments[0] + ":" + transformations[classFragments[1]] : className;
        }

        return transformations[className] || className;
      });
    });
    return [...classNames, ...(content.match(classRegex) || [])].filter((className, index, array) => array.indexOf(className) === index);
  }

}

var getWhitelist = ((projectTheme = {}, projectWhitelist = []) => {
  const components = renderComponentsToString(projectTheme); // eslint-disable-next-line no-useless-escape

  const componentWhitelist = components.match(classRegex) || [];
  return [...componentWhitelist, ...getThemeWhitelist(projectTheme), ...projectWhitelist].filter((className, index, array) => array.indexOf(className) === index);
});

exports.renderComponentsToString = renderComponentsToString;
exports.getThemeWhitelist = getThemeWhitelist;
exports.getWhitelist = getWhitelist;
exports.TailwindReactExtractor = TailwindReactExtractor;
